import 'dart:math';

import 'package:flutter/material.dart';

import 'package:snake/config/default_game_settings.dart';
import 'package:snake/models/settings/settings_game.dart';
import 'package:snake/models/settings/settings_global.dart';
import 'package:snake/utils/tools.dart';

class ParameterPainter extends CustomPainter {
  const ParameterPainter({
    required this.code,
    required this.value,
    required this.isSelected,
    required this.gameSettings,
    required this.globalSettings,
  });

  final String code;
  final String value;
  final bool isSelected;
  final GameSettings gameSettings;
  final GlobalSettings globalSettings;

  @override
  void paint(Canvas canvas, Size size) {
    // force square
    final double canvasSize = min(size.width, size.height);

    const Color borderColorEnabled = Colors.blue;
    const Color borderColorDisabled = Colors.white;

    // "enabled/disabled" border
    final paint = Paint();
    paint.style = PaintingStyle.stroke;
    paint.color = isSelected ? borderColorEnabled : borderColorDisabled;
    paint.strokeJoin = StrokeJoin.round;
    paint.strokeWidth = 10;
    canvas.drawRect(
        Rect.fromPoints(const Offset(0, 0), Offset(canvasSize, canvasSize)), paint);

    // content
    switch (code) {
      case DefaultGameSettings.parameterCodeLevel:
        paintLevelParameterItem(value, canvas, canvasSize);
        break;
      case DefaultGameSettings.parameterCodeSize:
        paintSizeParameterItem(value, canvas, canvasSize);
        break;
      default:
        printlog('Unknown parameter: $code/$value');
        paintUnknownParameterItem(value, canvas, canvasSize);
    }
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return false;
  }

  // "unknown" parameter -> simple block with text
  void paintUnknownParameterItem(
    final String value,
    final Canvas canvas,
    final double size,
  ) {
    final paint = Paint();
    paint.strokeJoin = StrokeJoin.round;
    paint.strokeWidth = 3;

    paint.color = Colors.grey;
    paint.style = PaintingStyle.fill;
    canvas.drawRect(Rect.fromPoints(const Offset(0, 0), Offset(size, size)), paint);

    final textSpan = TextSpan(
      text: '?\n$value',
      style: const TextStyle(
        color: Colors.black,
        fontSize: 18,
        fontWeight: FontWeight.bold,
      ),
    );
    final textPainter = TextPainter(
      text: textSpan,
      textDirection: TextDirection.ltr,
      textAlign: TextAlign.center,
    );
    textPainter.layout();
    textPainter.paint(
      canvas,
      Offset(
        (size - textPainter.width) * 0.5,
        (size - textPainter.height) * 0.5,
      ),
    );
  }

  void paintLevelParameterItem(
    final String value,
    final Canvas canvas,
    final double size,
  ) {
    const text = 'X';

    Color backgroundColor = Colors.grey;

    switch (value) {
      case DefaultGameSettings.levelValueEasy:
        backgroundColor = Colors.green;
        break;
      case DefaultGameSettings.levelValueMedium:
        backgroundColor = Colors.orange;
        break;
      case DefaultGameSettings.levelValueHard:
        backgroundColor = Colors.red;
        break;
      case DefaultGameSettings.levelValueNightmare:
        backgroundColor = Colors.grey;
        break;
      default:
        printlog('Wrong value for level parameter value: $value');
    }

    final paint = Paint();
    paint.strokeJoin = StrokeJoin.round;
    paint.strokeWidth = 3 / 100 * size;

    // Colored background
    paint.color = backgroundColor;
    paint.style = PaintingStyle.fill;
    canvas.drawRect(Rect.fromPoints(const Offset(0, 0), Offset(size, size)), paint);

    // centered text value
    final textSpan = TextSpan(
      text: text,
      style: TextStyle(
        color: Colors.black,
        fontSize: size / 2.6,
        fontWeight: FontWeight.bold,
      ),
    );
    final textPainter = TextPainter(
      text: textSpan,
      textDirection: TextDirection.ltr,
      textAlign: TextAlign.center,
    );
    textPainter.layout();
    textPainter.paint(
      canvas,
      Offset(
        (size - textPainter.width) * 0.5,
        (size - textPainter.height) * 0.5,
      ),
    );
  }

  void paintSizeParameterItem(
    final String value,
    final Canvas canvas,
    final double size,
  ) {
    const text = 'o';

    Color backgroundColor = Colors.grey;

    switch (value) {
      case DefaultGameSettings.sizeValueSmall:
        backgroundColor = Colors.green;
        break;
      case DefaultGameSettings.sizeValueMedium:
        backgroundColor = Colors.orange;
        break;
      case DefaultGameSettings.sizeValueLarge:
        backgroundColor = Colors.red;
        break;
      case DefaultGameSettings.sizeValueExtraLarge:
        backgroundColor = Colors.grey;
        break;
      default:
        printlog('Wrong value for size parameter value: $value');
    }

    final paint = Paint();
    paint.strokeJoin = StrokeJoin.round;
    paint.strokeWidth = 3;

    // Colored background
    paint.color = backgroundColor;
    paint.style = PaintingStyle.fill;
    canvas.drawRect(Rect.fromPoints(const Offset(0, 0), Offset(size, size)), paint);

    // centered text value
    final textSpan = TextSpan(
      text: text,
      style: TextStyle(
        color: Colors.black,
        fontSize: size / 2.6,
        fontWeight: FontWeight.bold,
      ),
    );
    final textPainter = TextPainter(
      text: textSpan,
      textDirection: TextDirection.ltr,
      textAlign: TextAlign.center,
    );
    textPainter.layout();
    textPainter.paint(
      canvas,
      Offset(
        (size - textPainter.width) * 0.5,
        (size - textPainter.height) * 0.5,
      ),
    );
  }
}
